unit PicClip;
{$R-}

interface

uses
  Windows, SysUtils, Classes, Graphics, Controls;

type
  TPicClip = class(TComponent)
  private
    FPicture: TBitmap;
    FRows: Integer;
    FCols: Integer;
    FCelWidth: Integer;
    FCelHeight: Integer;
    FCelMax: Integer;
  protected
    procedure Loaded; override;
    procedure SetPictureBitmap(const value: TBitmap);
    procedure SetCols(const value: Integer);
    procedure SetRows(const value: Integer);
    procedure SetDimension;
  public
    constructor Create(aOwner: TComponent); override;
    destructor Destroy; override;
    procedure GraphicCell(TmpBmp: TBitmap; nIndex: Integer);
  published
    Property Picture: TBitmap read FPicture write SetPictureBitmap stored True;
    Property Cols: Integer read FCols write SetCols stored True default 1;
    Property Rows: Integer read FRows write SetRows stored True default 1;
  end;

procedure Register;

implementation

constructor TPicClip.Create(aOwner: TComponent);
begin
  inherited Create(aOwner);
  FPicture := TBitmap.Create;
  FCelMax  := 0;
  FRows    := 1;
  FCols    := 1;
end;

destructor TPicClip.Destroy;
begin
  FPicture.Free;
  FPicture := nil;
  inherited Destroy;
end;

procedure TPicClip.Loaded;
begin
  inherited Loaded;
end;

procedure TPicClip.SetPictureBitmap(const value: TBitmap);
begin
  FPicture.Assign(value);
  SetDimension;
end;

procedure TPicClip.SetCols(const value: Integer);
begin
  if (value>0) then
  begin
    FCols := value;
    SetDimension;
  end;
end;

procedure TPicClip.SetRows(const value: Integer);
begin
  if (value>0) then
  begin
    FRows := value;
    SetDimension;
  end;
end;

procedure TPicClip.SetDimension;
begin
  FCelWidth  := FPicture.Width div FCols;
  FCelHeight := FPicture.Height div FRows;
  FCelMax    := FCols * FRows;
end;

procedure TPicClip.GraphicCell(TmpBmp: TBitmap; nIndex: Integer);
var
  x, y: Integer;
begin
  x :=0; y :=0;
  if ((nIndex > 0) and (nIndex <= FCelMax)) then
  begin
    TmpBmp.Width := FCelWidth;
    TmpBmp.Height := FCelHeight;
    if (nIndex<=FCols) then
    begin
      x := FCelWidth * (nIndex-1);
      y := 0;
    end
    else if (nIndex>FCols) then
    begin
      x := FCelWidth * (nIndex mod FCols);
      y := ((nIndex - 1) div FCols) * FCelHeight;
    end;
    SelectPalette(TmpBmp.Canvas.Handle, FPicture.Palette, True);
    RealizePalette(TmpBmp.Canvas.Handle);
    TmpBmp.Canvas.CopyRect(Rect(0, 0, FCelWidth, FCelHeight),
                           FPicture.Canvas,
                           Rect(x, y, x + FCelWidth, y + FCelHeight));
  end;
end;

procedure Register;
begin
  RegisterComponents('Sample', [TPicClip]);
end;

end.
